@@ -11,6 +11,7 @@
 
 #include <stdio.h>
 #include <math.h>
+#include <string.h>
 
 #include <gtk/gtksignal.h>
 #include <gtk/gtkmarshal.h>
@@ -50,7 +51,9 @@ static struct {
 } zoom_level_items[] = {
 	{ 1.0/4.0, "/commands/Zoom0104" },
 	{ 1.0/2.0, "/commands/Zoom0102" },
+	{ 3.0/4.0, "/commands/Zoom0304" },
 	{ 1.0/1.0, "/commands/Zoom0101" },
+	{ 3.0/2.0, "/commands/Zoom0302" },
 	{ 2.0/1.0, "/commands/Zoom0201" },
 	{ 4.0/1.0, "/commands/Zoom0401" },
 	{ 0.0, NULL }
@@ -120,6 +123,48 @@ sync_zoom_level_items(GgvControl *control)
 								 "state", "1", NULL);
 }
 
+static void
+sync_auto_fit_items(GgvControl *control)
+{
+	int i;
+	Bonobo_UIContainer container;
+	GgvPostScriptView *ps_view = control->priv->ps_view;
+	GtkWidget *gs;
+
+	container = bonobo_ui_component_get_container (control->priv->uic);
+	if (container == CORBA_OBJECT_NIL)
+		return;
+
+	gs = ggv_postscript_view_get_widget(ps_view);
+
+	switch(gtk_gs_get_zoom_mode(GTK_GS(gs))) {
+	case GTK_GS_ZOOM_ABSOLUTE:
+		bonobo_ui_component_set_prop(control->priv->uic,
+									 "/commands/AutoFitNone",
+									 "state", "1", NULL);
+		break;
+	case GTK_GS_ZOOM_FIT_WIDTH:
+		bonobo_ui_component_set_prop(control->priv->uic,
+									 "/commands/AutoFitWidth",
+									 "state", "1", NULL);
+		break;
+	case GTK_GS_ZOOM_FIT_PAGE:
+		bonobo_ui_component_set_prop(control->priv->uic,
+									 "/commands/AutoFitPage",
+									 "state", "1", NULL);
+		break;
+	default:
+		break;
+	}
+}
+
+static void
+sync_command_state(GgvControl *control)
+{
+	sync_auto_fit_items(control);
+	sync_zoom_level_items(control);
+}
+
 static void
 zoomable_set_frame_cb (BonoboZoomable *zoomable, GgvControl *control)
 {
@@ -145,8 +190,9 @@ zoomable_set_zoom_level_cb (BonoboZoomable *zoomable, float new_zoom_level,
 		(control->priv->ps_view);
 
 	if(control->priv->zoom_adj) {
-		gtk_adjustment_set_value(GTK_ADJUSTMENT(control->priv->zoom_adj),
-								 new_zoom_level*100.0);
+		if(fabs(GTK_ADJUSTMENT(control->priv->zoom_adj)->value - new_zoom_level*100.0) > 0.01)
+			gtk_adjustment_set_value(GTK_ADJUSTMENT(control->priv->zoom_adj),
+									 new_zoom_level*100.0);
 	}
 
 	bonobo_zoomable_report_zoom_level_changed
@@ -251,6 +297,7 @@ listener_ZoomLevel_cb(BonoboUIComponent *uic, const char *path,
 	BonoboZoomable *zoomable;
 	GgvControl *control;
 	const char *zl;
+	gint i;
 
 	g_return_if_fail(user_data != NULL);
 	g_return_if_fail(GGV_IS_CONTROL(user_data));
@@ -266,28 +313,46 @@ listener_ZoomLevel_cb(BonoboUIComponent *uic, const char *path,
 
 	zl = path + strlen("Zoom");
 
-	if(!strcmp(zl, "0104")) {
-		g_signal_emit_by_name (G_OBJECT (zoomable), "set_zoom_level",
-							   ((gfloat)1.0/4.0));
-	}
-	else if(!strcmp(zl, "0102")) {
-		g_signal_emit_by_name (G_OBJECT (zoomable), "set_zoom_level",
-							   ((gfloat)1.0/2.0));
-	}
-	else if(!strcmp(zl, "0101")) {
-		g_signal_emit_by_name (G_OBJECT (zoomable), "set_zoom_level",
-							   ((gfloat)1.0));
-	}
-	else if(!strcmp(zl, "0201")) {
-		g_signal_emit_by_name (G_OBJECT (zoomable), "set_zoom_level",
-							   ((gfloat)2.0/1.0));
-	}
-	else if(!strcmp(zl, "0401")) {
-		g_signal_emit_by_name (G_OBJECT (zoomable), "set_zoom_level",
-							   ((gfloat)4.0/1.0));
+	for(i = 0; zoom_level_items[i].path != NULL; i++) {
+		if(strstr(zoom_level_items[i].path, zl) != NULL) {
+			g_signal_emit_by_name (G_OBJECT (zoomable), "set_zoom_level",
+								   (gfloat)zoom_level_items[i].level);
+			break;
+		}
 	}
 }
 
+static void
+listener_AutoFitMode_cb(BonoboUIComponent *uic, const char *path,
+						Bonobo_UIComponent_EventType type, const char *state,
+						gpointer user_data)
+{
+	GgvControl *control;
+	GtkGS *gs;
+	gint i;
+
+	g_return_if_fail(user_data != NULL);
+	g_return_if_fail(GGV_IS_CONTROL(user_data));
+
+	if(type != Bonobo_UIComponent_STATE_CHANGED)
+		return;
+
+	if(!state || !atoi(state))
+		return;
+
+	control = GGV_CONTROL(user_data);
+	gs = GTK_GS(ggv_postscript_view_get_widget(control->priv->ps_view));
+
+	if(!strcmp(path, "AutoFitNone"))
+		gtk_gs_set_zoom_mode(gs, GTK_GS_ZOOM_ABSOLUTE);
+	if(!strcmp(path, "AutoFitWidth"))
+		gtk_gs_set_zoom_mode(gs, GTK_GS_ZOOM_FIT_WIDTH);
+	if(!strcmp(path, "AutoFitPage"))
+		gtk_gs_set_zoom_mode(gs, GTK_GS_ZOOM_FIT_PAGE);
+
+	gtk_widget_unref(GTK_WIDGET(gs));
+}
+
 static gboolean
 ggv_control_button_press_event(GtkWidget *widget, GdkEventButton *event,
 							   gpointer data)
@@ -310,7 +375,7 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 	GtkGSOrientation orientation = gtk_gs_get_orientation(gs);
 	GgvPostScriptView *ps_view = control->priv->ps_view;
 
-	/*ugh. the possibilities! */
+	/* ugh. the possibilities! */
 	switch (key) {
 	case GDK_space:
 		switch (orientation) {
@@ -366,16 +431,24 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 		}
 		break;
 	case GDK_Left:
-		gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_LEFT, FALSE);
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_LEFT, FALSE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
 		break;
 	case GDK_Right:
-		gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_RIGHT, FALSE);
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_RIGHT, FALSE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 		break;
 	case GDK_Up:
-		gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_UP, FALSE);
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_UP, FALSE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
 		break;
 	case GDK_Down:
-		gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_DOWN, FALSE);
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_DOWN, FALSE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 		break;
 	case GDK_Page_Up:
 		ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
@@ -480,6 +553,7 @@ _set_zoom_items_sensitivity(GgvControl *control, BonoboUIComponent *uic, gboolea
 {
 	gchar *prop_val = sens?"1":"0";
 	Bonobo_UIContainer container;
+	gint i;
 
 	container = bonobo_ui_component_get_container (uic);
 	if (container == CORBA_OBJECT_NIL)
@@ -496,16 +570,10 @@ _set_zoom_items_sensitivity(GgvControl *control, BonoboUIComponent *uic, gboolea
 								 "sensitive", prop_val, NULL);
 	bonobo_ui_component_set_prop(uic, "/commands/ZoomToFitWidth",
 								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/Zoom0104",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/Zoom0102",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/Zoom0101",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/Zoom0201",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/Zoom0401",
-								 "sensitive", prop_val, NULL);
+	for(i = 0; zoom_level_items[i].path != NULL; i++) {
+		bonobo_ui_component_set_prop(uic, zoom_level_items[i].path,
+									 "sensitive", prop_val, NULL);
+	}
 	bonobo_ui_component_thaw(uic, NULL);
 }
 
@@ -517,6 +585,33 @@ set_zoom_items_sensitivity(GgvControl *control, gboolean sens)
 		_set_zoom_items_sensitivity(control, control->priv->popup_uic, sens);
 }
 
+static void
+set_auto_fit_items_sensitivity(GgvControl *control, gboolean sens)
+{
+	gchar *prop_val = sens?"1":"0";
+	Bonobo_UIContainer container;
+
+	container = bonobo_ui_component_get_container (control->priv->uic);
+	if (container == CORBA_OBJECT_NIL)
+		return;
+
+	bonobo_ui_component_freeze(control->priv->uic, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitNone",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitWidth",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitPage",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_thaw(control->priv->uic, NULL);	
+}
+
+static void
+set_command_items_sensitivity(GgvControl *control, gboolean sens)
+{
+	set_auto_fit_items_sensitivity(control, sens);
+	set_zoom_items_sensitivity(control, sens);
+}
+
 static BonoboUIVerb ggv_control_verbs[] = {
 		BONOBO_UI_VERB ("ZoomIn",         verb_ZoomIn_cb),
 		BONOBO_UI_VERB ("ZoomOut",        verb_ZoomOut_cb),
@@ -552,7 +647,7 @@ ggv_control_create_ui (GgvControl *control)
 
 	klass = GGV_CONTROL_CLASS(G_OBJECT_GET_CLASS(control));
 
-	bonobo_ui_util_set_ui(control->priv->uic, NULL, "ggv-control-ui.xml",
+	bonobo_ui_util_set_ui(control->priv->uic, DATADIR, "ggv-control-ui.xml",
 						  "GGV", NULL);
 	pixbuf = gdk_pixbuf_new_from_file(GNOMEICONDIR "/ggv/fitwidth.png", NULL);
 	if(pixbuf != NULL) {
@@ -567,17 +662,26 @@ ggv_control_create_ui (GgvControl *control)
 								 
 	control->priv->popup_uic = bonobo_control_get_popup_ui_component(BONOBO_CONTROL(control));
 	bonobo_ui_util_set_ui(control->priv->popup_uic, NULL, "ggv-postscript-view-ui.xml", "GGV", NULL);
-	bonobo_ui_util_set_ui(control->priv->popup_uic, NULL, "ggv-control-ui.xml", "GGV", NULL);
+	bonobo_ui_util_set_ui(control->priv->popup_uic, DATADIR, "ggv-control-ui.xml", "GGV", NULL);
 	
 	for(i = 0; zoom_level_items[i].path; i++) {
 		bonobo_ui_component_add_listener(control->priv->uic,
 										 zoom_level_items[i].path + strlen("/commands/"),
 										 listener_ZoomLevel_cb, control);
 	}
+	bonobo_ui_component_add_listener(control->priv->uic,
+									 "AutoFitNone",
+									 listener_AutoFitMode_cb, control);
+	bonobo_ui_component_add_listener(control->priv->uic,
+									 "AutoFitWidth",
+									 listener_AutoFitMode_cb, control);
+	bonobo_ui_component_add_listener(control->priv->uic,
+									 "AutoFitPage",
+									 listener_AutoFitMode_cb, control);
 
-	set_zoom_items_sensitivity(control, ggv_postscript_view_get_page_count(control->priv->ps_view) > 0);
+	set_command_items_sensitivity(control, ggv_postscript_view_get_page_count(control->priv->ps_view) > 0);
 
-	sync_zoom_level_items(control);
+	sync_command_state(control);
 
 	bonobo_ui_component_add_verb_list_with_data(control->priv->uic,
 												ggv_control_verbs,
@@ -711,7 +815,7 @@ ps_view_property_changed_handler(BonoboListener    *listener,
         GgvControl *control = GGV_CONTROL(data);
 
         if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:page")) {
-			set_zoom_items_sensitivity(control, BONOBO_ARG_GET_INT(any) != -1);
+			set_command_items_sensitivity(control, BONOBO_ARG_GET_INT(any) != -1);
         }
 }
 
@@ -765,6 +869,8 @@ ggv_control_construct (GgvControl *control, GgvPostScriptView *ps_view)
 					 G_CALLBACK(scrollbar_button_release_event),
 					 view);
 	gtk_container_add(GTK_CONTAINER(control->priv->root), view);
+	/* unref the GtkGS acquired from GgvPostScriptView */
+	gtk_widget_unref(view);
 	
 	bonobo_control_construct (BONOBO_CONTROL (control), control->priv->root);
 
