@@ -56,7 +56,7 @@ struct _GgvPostScriptViewPrivate {
 
 	gboolean               zoom_fit;
 
-	gboolean pane_auto_jump;
+	gboolean pane_auto_jump, page_flip;
 };
 
 struct _GgvPostScriptViewClassPrivate {
@@ -158,7 +158,7 @@ set_file_items_sensitivity(GgvPostScriptView *ps_view, gboolean sens)
 	if (container == CORBA_OBJECT_NIL)
 		return;
 
-	prop_val = (sens && !GTK_GS_IS_PDF(ps_view->priv->gs))?"1":"0";
+	prop_val = sens?"1":"0";
 
 	bonobo_ui_component_freeze(ps_view->priv->uic, NULL);
 	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/FileSaveMarked",
@@ -1744,6 +1744,19 @@ impl_GNOME_GGV_PostScriptView_reload(PortableServer_Servant servant,
 	notify_status_change(ps_view);
 }
 
+static void
+impl_GNOME_GGV_PostScriptView_close(PortableServer_Servant servant,
+									CORBA_Environment *ev)
+{
+	GgvPostScriptView *ps_view;
+
+	ps_view = GGV_POSTSCRIPT_VIEW(bonobo_object_from_servant(servant));
+	gtk_gs_load(GTK_GS(ps_view->priv->gs), NULL);
+	notify_page_count_change(ps_view);
+	notify_page_change(ps_view);
+	notify_status_change(ps_view);
+}
+
 static void
 ggv_postscript_view_class_init(GgvPostScriptViewClass *klass)
 {
@@ -1763,6 +1776,7 @@ ggv_postscript_view_class_init(GgvPostScriptViewClass *klass)
 	epv->getDocument = impl_GNOME_GGV_PostScriptView_getDocument;
 	epv->getPages = impl_GNOME_GGV_PostScriptView_getPages;
 	epv->reload = impl_GNOME_GGV_PostScriptView_reload;
+	epv->close = impl_GNOME_GGV_PostScriptView_close;
 
 	/* this seems as a nice place to load the prefs */
 	ggv_prefs_load();
@@ -1877,7 +1891,7 @@ view_realized_cb(GtkGS *gs, gpointer data)
 
 	cursor = cursor_get(GTK_WIDGET(gs)->window, CURSOR_HAND_OPEN);
 	gdk_window_set_cursor(GTK_WIDGET(gs)->window, cursor);
-	gdk_cursor_destroy(cursor);
+	gdk_cursor_unref(cursor);
 }
 
 static void
@@ -1915,10 +1929,14 @@ ggv_postscript_view_prefs_changed(GConfClient *client, guint cnxn_id,
 	}
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/watch_doc"))
 		gtk_gs_set_watch_file(gs, gconf_value_get_bool(entry->value));
+	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/scrollstep"))
+		gtk_gs_set_scroll_step(gs, gconf_value_get_float(entry->value));
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/show_scroll_rect"))
 		gtk_gs_set_show_scroll_rect(gs, gconf_value_get_bool(entry->value));
 	else if(!strcmp(entry->key, "/apps/ggv/control/autojump"))
 	   ps_view->priv->pane_auto_jump = gconf_value_get_bool(entry->value);
+	else if(!strcmp(entry->key, "/apps/ggv/control/pageflip"))
+	   ps_view->priv->page_flip = gconf_value_get_bool(entry->value);
 }
 
 static gboolean
@@ -1935,19 +1953,28 @@ sidebar_key_press_event(GtkWidget *widget, GdkEventKey *event,
 
 	switch(event->keyval) {
 	case GDK_Left:
-		gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_LEFT, FALSE);
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_LEFT, FALSE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
 		break;
 	case GDK_Right:
-		gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_RIGHT, FALSE);
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_RIGHT, FALSE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 		break;
 	case GDK_Up:
-		gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_UP, FALSE);
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_UP, FALSE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
 		break;
 	case GDK_Down:
-		gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_DOWN, FALSE);
+		if(!gtk_gs_scroll_step(gs, GTK_SCROLL_STEP_DOWN, FALSE) &&
+		   ggv_postscript_view_get_page_flip(ps_view))
+			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 		break;
 	default:
 		return FALSE;
+		break;
 	}
 	return TRUE;
 }
@@ -2078,9 +2105,14 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 	g_signal_connect(G_OBJECT(ggv_sidebar_get_checklist(ps_view->priv->sidebar)),
 					 "key_press_event", G_CALLBACK(sidebar_key_press_event), ps_view);
 
-	ps_view->priv->pane_auto_jump = gconf_client_get_bool(gtk_gs_defaults_gconf_client(),
-														  "/apps/ggv/control/autojump",
-														  NULL);
+	ps_view->priv->pane_auto_jump =
+		gconf_client_get_bool(gtk_gs_defaults_gconf_client(),
+							  "/apps/ggv/control/autojump",
+							  NULL);
+	ps_view->priv->page_flip =
+		gconf_client_get_bool(gtk_gs_defaults_gconf_client(),
+							  "/apps/ggv/control/pageflip",
+							  NULL);
 	return ps_view;
 }
 
@@ -2116,3 +2148,15 @@ ggv_postscript_view_set_popup_ui_component(GgvPostScriptView *ps_view,
 	ps_view->priv->popup_uic = uic;
 	_set_page_items_sensitivity(ps_view, uic);
 }
+
+gboolean
+ggv_postscript_view_get_auto_jump(GgvPostScriptView *ps_view)
+{
+	return ps_view->priv->pane_auto_jump;
+}
+
+gboolean
+ggv_postscript_view_get_page_flip(GgvPostScriptView *ps_view)
+{
+	return ps_view->priv->page_flip;
+}
