@@ -253,9 +253,12 @@ ggv_sidebar_page_changed(GgvSidebar *sidebar, gint page)
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(sidebar->priv->checklist));
 	g_snprintf(path_str, 15, "%d", page);
 	path = gtk_tree_path_new_from_string(path_str);
-	if(!gtk_tree_selection_path_is_selected(sel, path))
+	if(!gtk_tree_selection_path_is_selected(sel, path)) {
 		gtk_tree_selection_select_path(sel, path);
-	gtk_tree_path_free(path);	
+	}
+	gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(sidebar->priv->checklist),
+								 path, NULL, FALSE, 0.0, 0.0);
+	gtk_tree_path_free(path);
 }
 
 gint *
@@ -286,11 +289,13 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	g_object_ref(G_OBJECT(sidebar->priv->toggle_tips));
 	gtk_object_sink(GTK_OBJECT(sidebar->priv->toggle_tips));
 
-	sidebar->priv->coordinates = gtk_label_new(" ");
+	sidebar->priv->coordinates = gtk_entry_new();
+	gtk_widget_set_size_request(sidebar->priv->coordinates, 0, -1);
+	gtk_editable_set_editable(GTK_EDITABLE(sidebar->priv->coordinates), FALSE);
 	gtk_widget_show(sidebar->priv->coordinates);
 	gtk_box_pack_start(GTK_BOX(sidebar->priv->root),
 					   sidebar->priv->coordinates,
-					   FALSE, TRUE, 2);
+					   FALSE, FALSE, 2);
 
 	hbox = gtk_hbox_new(TRUE, 2);
 	gtk_widget_show(hbox);
@@ -303,7 +308,7 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->toggle_all,
 						 _("Toggle marked state of all pages"),
 						 _("Toggle marked state of all pages: previously "
-						   "marked pages will be unmarked and umarked ones "
+						   "marked pages will be unmarked and unmarked ones "
 						   "will become marked."));
 	gtk_widget_show(sidebar->priv->toggle_all);
 	gtk_box_pack_start(GTK_BOX(hbox), sidebar->priv->toggle_all, FALSE, TRUE, 0);
@@ -316,7 +321,7 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->toggle_odd,
 						 _("Toggle marked state of odd pages"),
 						 _("Toggle marked state of odd pages: previously "
-						   "marked odd pages will be unmarked and umarked "
+						   "marked odd pages will be unmarked and unmarked "
 						   "ones will become marked."));
 	gtk_button_set_relief(GTK_BUTTON(sidebar->priv->toggle_odd), GTK_RELIEF_NONE);
 	gtk_widget_show(sidebar->priv->toggle_odd);
@@ -330,7 +335,7 @@ ggv_sidebar_construct (GgvSidebar *sidebar, GgvPostScriptView *ps_view)
 	gtk_tooltips_set_tip(sidebar->priv->toggle_tips, sidebar->priv->toggle_even,
 						 _("Toggle marked state of even pages"),
 						 _("Toggle marked state of even pages: previously "
-						   "marked even pages will be unmarked and umarked "
+						   "marked even pages will be unmarked and unmarked "
 						   "ones will become marked."));
 	gtk_button_set_relief(GTK_BUTTON(sidebar->priv->toggle_even), GTK_RELIEF_NONE);
 	gtk_widget_show(sidebar->priv->toggle_even);
@@ -396,7 +401,7 @@ ggv_sidebar_update_coordinates(GgvSidebar *sidebar, gfloat xcoord, gfloat ycoord
 		clabel = g_strdup_printf("%.2f , %.2f",
 								 xcoord*ggv_unit_factors[ggv_unit_index],
 								 ycoord*ggv_unit_factors[ggv_unit_index]);
-		gtk_label_set_text(GTK_LABEL(sidebar->priv->coordinates), clabel);
+		gtk_entry_set_text(GTK_LABEL(sidebar->priv->coordinates), clabel);
 		g_free(clabel);
 	}
 }
