@@ -33,7 +33,7 @@
 static void gtk_check_list_init          (GtkCheckList      *check_list);
 static void gtk_check_list_class_init    (GtkCheckListClass *class);
 
-static GtkCListClass *parent_class = NULL;
+static GtkTreeViewClass *parent_class = NULL;
 
 GType
 gtk_check_list_get_type (void)
