@@ -127,6 +127,8 @@ The DONE message indicates that ghostscript has finished processing.
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <stdio.h>
+#include <math.h>
+
 #include "gtkgs.h"
 #include "ggvutils.h"
 #include "ps.h"
@@ -348,6 +350,8 @@ gtk_gs_init (GtkGS *gs)
         gs->respect_eof = gtk_gs_defaults_get_respect_eof();
         gs->watch_doc = gtk_gs_defaults_get_watch_doc();
         gs->show_scroll_rect = gtk_gs_defaults_get_show_scroll_rect();
+        gs->scroll_step = gtk_gs_defaults_get_scroll_step();
+        gs->zoom_mode = gtk_gs_defaults_get_zoom_mode();
 
         gs->scroll_start_x = gs->scroll_start_y = -1;
 
@@ -508,12 +512,6 @@ gtk_gs_destroy (GtkObject *object)
 		g_free(gs->input_buffer);
                 gs->input_buffer = NULL;
         }
-
-        if(gs->gconf_notify_id != -1) {
-                gconf_client_notify_remove(gtk_gs_defaults_gconf_client(),
-                                           gs->gconf_notify_id);
-                gs->gconf_notify_id = -1;
-        }
         if(gs->hadj) {
                 g_signal_handlers_disconnect_matched (G_OBJECT (gs->hadj),
                                                       G_SIGNAL_MATCH_DATA,
@@ -683,8 +681,13 @@ gtk_gs_size_allocate (GtkWidget     *widget,
         /* 
          * update the adjustment if necessary (ie. a resize);
          */
-        if (GTK_WIDGET_REALIZED(gs)) {
-		gtk_gs_munge_adjustments (gs);
+        if (gs->zoom_mode == GTK_GS_ZOOM_ABSOLUTE) {
+                if (GTK_WIDGET_REALIZED(gs)) {
+                        gtk_gs_munge_adjustments (gs);
+                }
+        }
+        else {
+                gtk_gs_set_zoom(gs, 0.0);
         }
 
         gtk_gs_goto_page(gs, gs->current_page);
@@ -1081,7 +1084,6 @@ input (gpointer data, gint source, GdkInputCondition condition)
         }
 }
 
-
 static int
 start_interpreter (GtkGS *gs)
 {
@@ -1111,7 +1113,7 @@ start_interpreter (GtkGS *gs)
         for(i = 0; i < NUM_GS_ARGS && gs_args[i]; i++, argc++)
                 argv[argc] = gs_args[i];
 
-        if(gs->antialiased ) {
+        if(gs->antialiased) {
                 if(strlen(gtk_gs_defaults_get_alpha_parameters()) == 0)
                         alpha_args = g_strsplit(ALPHA_PARAMS, " ",
                                                 NUM_ALPHA_ARGS);
@@ -1237,17 +1239,17 @@ start_interpreter (GtkGS *gs)
 static void
 stop_interpreter (GtkGS *gs)
 {
-        if (gs->interpreter_pid >= 0) {
+        if (gs->interpreter_pid > 0) {
                 int status = 0;
                 kill(gs->interpreter_pid, SIGTERM);
                 wait(&status);
+		gs->interpreter_pid = -1;
                 if(status == 1) {
                         gtk_gs_cleanup(gs);
                         gs->gs_status = _("Interpreter failed.");
                         g_signal_emit_by_name(G_OBJECT(gs), "interpreter_error",
                                               status);
                 }
-		gs->interpreter_pid = -1;
 	}
 
 	if (gs->interpreter_input >= 0) {
@@ -1404,8 +1406,8 @@ check_pdf (GtkGS *gs)
                 }
                 close(fd);
 		fname = ggv_quote_filename (filename);
-		cmd = g_strdup_printf (gtk_gs_defaults_get_scan_pdf_cmd(),
-                                       fname, filename_dsc);
+		cmd = g_strdup_printf (gtk_gs_defaults_get_convert_pdf_cmd(),
+                                       filename_dsc, fname);
 		g_free (fname);
 		/* this command (sometimes?) prints error messages to stdout! */
 		cmdline = g_strdup_printf ("%s >%s 2>&1", cmd, filename_err);
@@ -1419,7 +1421,7 @@ check_pdf (GtkGS *gs)
 		} else {
 			/* report error */
 			g_snprintf (buf, 1024,
-				    _("Error while scanning pdf file %s:\n"),
+				    _("Error while converting pdf file %s:\n"),
                                     filename);
 			gtk_gs_emit_error_msg (gs, buf);
 
@@ -1582,7 +1584,7 @@ GType gtk_gs_get_type (void)
                                         NULL, /* class_data */
                                         sizeof (GtkGS),
                                         0, /* n_preallocs */
-                                        (GtkObjectInitFunc) gtk_gs_init
+                                        (GInstanceInitFunc) gtk_gs_init
                                         };
 
                 gs_type = g_type_register_static(gtk_widget_get_type (),
@@ -1603,7 +1605,7 @@ gtk_gs_new (GtkAdjustment *hadj, GtkAdjustment *vadj)
         g_return_val_if_fail(hadj != NULL, NULL);
         g_return_val_if_fail(vadj != NULL, NULL);
 
-	gs = gtk_type_new(gtk_gs_get_type());
+	gs = (GtkGS *)gtk_type_new(gtk_gs_get_type());
 
         gtk_gs_set_adjustments(gs, hadj, vadj);
 
@@ -1714,6 +1716,7 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
 	if (fname == NULL) {
                 if(gdk_window_is_visible(gs->pstarget))
                         gdk_window_hide(gs->pstarget);
+                gs->gs_status = "";
 		return FALSE;
 	}
 
@@ -1945,6 +1948,9 @@ gtk_gs_goto_page (GtkGS *gs, gint page)
                    the last page of the file was displayed. In that
                    case, ggv restarts GS again and the first page is displayed.
                 */
+                if (page == gs->current_page && gs->changed)
+                        return TRUE;
+
                 if(!GTK_WIDGET_REALIZED(gs))
                         return FALSE;                   
 
@@ -2137,14 +2143,28 @@ gtk_gs_set_zoom (GtkGS *gs, gfloat zoom) {
         g_return_if_fail(gs != NULL);
         g_return_if_fail(GTK_IS_GS(gs));
 
-        if (gs->zoom_factor != zoom) { 
+        switch(gs->zoom_mode) {
+        case GTK_GS_ZOOM_FIT_WIDTH:
+                zoom = gtk_gs_zoom_to_fit(gs, TRUE);
+                break;
+        case GTK_GS_ZOOM_FIT_PAGE:
+                zoom = gtk_gs_zoom_to_fit(gs, FALSE);                
+                break;
+        case GTK_GS_ZOOM_ABSOLUTE:
+        default:
+                break;
+        }
+        if(zoom < ggv_zoom_levels[0])
+                zoom = ggv_zoom_levels[0];
+        else if(zoom > ggv_zoom_levels[ggv_max_zoom_levels])
+                zoom = ggv_zoom_levels[ggv_max_zoom_levels];
+        if (fabs(gs->zoom_factor - zoom) > 0.001) { 
                 gs->zoom_factor = zoom;
                 if (GTK_WIDGET_REALIZED(gs)) 
                         set_up_page(gs);
                 gs->changed = TRUE;
+                gtk_widget_queue_resize (&(gs->widget));
         }
-	gtk_widget_queue_resize (&(gs->widget));
-
 }
 
 gfloat
@@ -2494,9 +2514,9 @@ gtk_gs_scroll_step(GtkGS *gs, GtkScrollType direction, gboolean dowrap)
 
         /* To make code more readable, we make a macro */
 #define ADVANCE_TOWARDS_LOWER(adj) \
-        (adj->value -= 0.75 * (adj->page_size))
+        (adj->value -= gs->scroll_step * (adj->page_size))
 #define ADVANCE_TOWARDS_UPPER(adj) \
-        (adj->value += 0.75 * (adj->page_size))
+        (adj->value += gs->scroll_step * (adj->page_size))
 
 #define ADVANCE_STEP(adj,dir) \
         (dir?ADVANCE_TOWARDS_UPPER(adj):ADVANCE_TOWARDS_LOWER(adj))
@@ -2580,7 +2600,7 @@ gtk_gs_get_postscript(GtkGS *gs, gint *pages)
         gboolean free_pages = FALSE;
 
         if(pages == NULL) {
-                if(gs->doc->epsf) {
+                if(!gs->structured_doc) {
                         FILE *f;
                         struct stat sb;
 
@@ -2691,3 +2711,30 @@ gtk_gs_set_adjustments (GtkGS         *gs,
                 gtk_gs_munge_adjustments(gs);
 }
 
+
+void
+gtk_gs_set_scroll_step(GtkGS *gs, gfloat scroll_step)
+{
+	gs->scroll_step = scroll_step;
+}
+
+gfloat
+gtk_gs_get_scroll_step(GtkGS *gs)
+{
+	return gs->scroll_step;
+}
+
+void
+gtk_gs_set_zoom_mode(GtkGS *gs, GtkGSZoomMode zoom_mode)
+{
+        if(zoom_mode != gs->zoom_mode) {
+                gs->zoom_mode = zoom_mode;
+                gtk_gs_set_zoom(gs, 1.0);
+        }
+}
+
+GtkGSZoomMode
+gtk_gs_get_zoom_mode(GtkGS *gs)
+{
+        return gs->zoom_mode;
+}
