@@ -19,6 +19,9 @@
 
 #ifdef __cplusplus
 extern "C" {
+#if 0
+} /* let's fool emacs */
+#endif /* 0 */
 #endif /* __cplusplus */
         
 #define GTK_GS(obj)         GTK_CHECK_CAST (obj, gtk_gs_get_type (), GtkGS)
@@ -40,6 +43,13 @@ typedef enum
         GTK_GS_ORIENTATION_LANDSCAPE = 1,
 } GtkGSOrientation;
 
+typedef enum
+{
+        GTK_GS_ZOOM_ABSOLUTE = 0,
+        GTK_GS_ZOOM_FIT_WIDTH = 1,
+        GTK_GS_ZOOM_FIT_PAGE = 2
+} GtkGSZoomMode;
+
 struct _GtkGS
 {
         GtkWidget widget;                    /* the main widget */
@@ -53,6 +63,8 @@ struct _GtkGS
         gint scroll_width, scroll_height;
         gboolean show_scroll_rect;
 
+        GtkGSZoomMode zoom_mode;
+
 	GdkPixmap *bpixmap;			/* Backing pixmap */
 	int        use_bpixmap;
 
@@ -80,6 +92,7 @@ struct _GtkGS
 	gboolean    busy;			/* Is gs busy drawing? */
 	gboolean    changed;			/* Anything changed since setup */
         gfloat      zoom_factor;
+        gfloat      scroll_step;
         gint        current_page;
         gboolean    structured_doc;
         gboolean    loaded;
@@ -112,8 +125,6 @@ struct _GtkGS
         gint        fallback_orientation;  /* Orientation to use if override */
         gint        real_orientation;      /* Real orientation from the document */
 
-        guint      gconf_notify_id;
-
         const gchar *gs_status;            /* GtkGS status */
 };
 
@@ -180,6 +191,8 @@ void         gtk_gs_set_default_size(GtkGS *gs, gint size);
 gint         gtk_gs_get_default_size(GtkGS *gs);
 void         gtk_gs_set_zoom(GtkGS *gs, gfloat zoom);
 gfloat       gtk_gs_get_zoom(GtkGS *gs);
+void         gtk_gs_set_scroll_step(GtkGS *gs, gfloat scroll_step);
+gfloat       gtk_gs_get_scroll_step(GtkGS *gs);
 gfloat       gtk_gs_zoom_to_fit(GtkGS *gs, gboolean fit_width);
 void         gtk_gs_set_center(GtkGS *gs, gfloat hval, gfloat vval);
 gint         gtk_gs_get_orientation(GtkGS *gs);
@@ -202,6 +215,9 @@ gboolean     gtk_gs_get_show_scroll_rect(GtkGS *gs);
 void         gtk_gs_start_scroll(GtkGS *gs);
 void         gtk_gs_end_scroll(GtkGS *gs);
 
+void          gtk_gs_set_zoom_mode(GtkGS *gs, GtkGSZoomMode zoom_mode);
+GtkGSZoomMode gtk_gs_get_zoom_mode(GtkGS *gs);
+
 /* utility functions */
 gint     gtk_gs_get_size_index(gchar *string, GtkGSPaperSize *size);
 
@@ -212,6 +228,7 @@ gchar   *gtk_gs_get_postscript(GtkGS *gs, gint *pages);
                                         GTK_GS(gs)->gs_filename_unc : \
                                         GTK_GS(gs)->gs_filename)
 #define GTK_GS_IS_PDF(gs)              (GTK_GS(gs)->gs_filename_dsc != NULL)
+#define GTK_GS_IS_STRUCTURED_DOC(gs)   (GTK_GS(gs)->structured_doc)
 
 #ifdef __cplusplus
 }
