@@ -267,7 +267,7 @@ struct document *psscan (FILE *file, int respect_eof)
 
 /* Jake Hamby patch 18/3/98 */
 
-	sscanf(line, "%*s %s", text);
+	sscanf(line, "%*s %256s", text);
 	/*doc->epsf = iscomment(text, "EPSF-");*/
 	doc->epsf = iscomment(text, "EPSF"); /* Hamby - This line changed */
 	doc->beginheader = position;
@@ -308,7 +308,7 @@ struct document *psscan (FILE *file, int respect_eof)
 	} else if (doc->date == NULL && iscomment(line+2, "CreationDate:")) {
 	    doc->date = gettextline(line+length("%%CreationDate:"));
 	} else if (bb_set == NONE && iscomment(line+2, "BoundingBox:")) {
-	    sscanf(line+length("%%BoundingBox:"), "%s", text);
+	    sscanf(line+length("%%BoundingBox:"), "%256s", text);
 	    if (strcmp(text, "(atend)") == 0) {
 		bb_set = ATEND;
 	    } else {
@@ -340,7 +340,7 @@ struct document *psscan (FILE *file, int respect_eof)
 	    }
 	} else if (orientation_set == NONE &&
 		   iscomment(line+2, "Orientation:")) {
-	    sscanf(line+length("%%Orientation:"), "%s", text);
+	    sscanf(line+length("%%Orientation:"), "%256s", text);
 	    if (strcmp(text, "(atend)") == 0) {
 		orientation_set = ATEND;
 	    } else if (strcmp(text, "Portrait") == 0) {
@@ -355,7 +355,7 @@ struct document *psscan (FILE *file, int respect_eof)
                     orientation_set = 1;
             }
 	} else if (page_order_set == NONE && iscomment(line+2, "PageOrder:")) {
-	    sscanf(line+length("%%PageOrder:"), "%s", text);
+	    sscanf(line+length("%%PageOrder:"), "%256s", text);
 	    if (strcmp(text, "(atend)") == 0) {
 		page_order_set = ATEND;
 	    } else if (strcmp(text, "Ascend") == 0) {
@@ -369,7 +369,7 @@ struct document *psscan (FILE *file, int respect_eof)
 		page_order_set = 1;
 	    }
 	} else if (pages_set == NONE && iscomment(line+2, "Pages:")) {
-	    sscanf(line+length("%%Pages:"), "%s", text);
+	    sscanf(line+length("%%Pages:"), "%256s", text);
 	    if (strcmp(text, "(atend)") == 0) {
 		pages_set = ATEND;
 	    } else {
@@ -568,7 +568,7 @@ struct document *psscan (FILE *file, int respect_eof)
 		/* Do nothing */
 	    } else if (doc->default_page_orientation == NONE &&
 		iscomment(line+2, "PageOrientation:")) {
-		sscanf(line+length("%%PageOrientation:"), "%s", text);
+		sscanf(line+length("%%PageOrientation:"), "%256s", text);
 		if (strcmp(text, "Portrait") == 0) {
 		    doc->default_page_orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
@@ -695,7 +695,7 @@ struct document *psscan (FILE *file, int respect_eof)
 		/* Do nothing */
 	    } else if (doc->default_page_orientation == NONE &&
 		iscomment(line+2, "PageOrientation:")) {
-		sscanf(line+length("%%PageOrientation:"), "%s", text);
+		sscanf(line+length("%%PageOrientation:"), "%256s", text);
 		if (strcmp(text, "Portrait") == 0) {
 		    doc->default_page_orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
@@ -837,7 +837,7 @@ continuepage:
 		/* Do nothing */
 	    } else if (doc->pages[doc->numpages].orientation == NONE &&
 		iscomment(line+2, "PageOrientation:")) {
-		sscanf(line+length("%%PageOrientation:"), "%s", text);
+		sscanf(line+length("%%PageOrientation:"), "%256s", text);
 		if (strcmp(text, "Portrait") == 0) {
 		    doc->pages[doc->numpages].orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
@@ -872,7 +872,7 @@ continuepage:
 		g_free(cp);
 	    } else if ((page_bb_set == NONE || page_bb_set == ATEND) &&
 		       iscomment(line+2, "PageBoundingBox:")) {
-		sscanf(line+length("%%PageBoundingBox:"), "%s", text);
+		sscanf(line+length("%%PageBoundingBox:"), "%256s", text);
 		if (strcmp(text, "(atend)") == 0) {
 		    page_bb_set = ATEND;
 		} else {
@@ -986,7 +986,7 @@ continuepage:
 	    }
 	} else if (orientation_set == ATEND &&
 		   iscomment(line+2, "Orientation:")) {
-	    sscanf(line+length("%%Orientation:"), "%s", text);
+	    sscanf(line+length("%%Orientation:"), "%256s", text);
 	    if (strcmp(text, "Portrait") == 0) {
 		doc->orientation = GTK_GS_ORIENTATION_PORTRAIT;
 	    } else if (strcmp(text, "Landscape") == 0) {
@@ -996,7 +996,7 @@ continuepage:
                     doc->orientation = GTK_GS_ORIENTATION_SEASCAPE;
             }
 	} else if (page_order_set == ATEND && iscomment(line+2, "PageOrder:")) {
-	    sscanf(line+length("%%PageOrder:"), "%s", text);
+	    sscanf(line+length("%%PageOrder:"), "%256s", text);
 	    if (strcmp(text, "Ascend") == 0) {
 		doc->pageorder = ASCEND;
 	    } else if (strcmp(text, "Descend") == 0) {
@@ -1278,7 +1278,7 @@ readline(lineP, size, fp, positionP, line_lenP)
    else if IS_BEGIN("Data:")      {
 	text[0] = '\0';
 	strcpy(save, line+7);
-	if (sscanf(line+length("%%BeginData:"), "%d %*s %s", &num, text) >= 1) {
+	if (sscanf(line+length("%%BeginData:"), "%d %*s %256s", &num, text) >= 1) {
 	    if (strcmp(text, "Lines") == 0) {
 		for (i=0; i < num; i++) {
 		    cp = fgets(line, size, fp);
@@ -1335,12 +1335,19 @@ readline(line, size, fp, position, line_len)
     char *cp;
     unsigned int num;
     unsigned int nbytes;
-    int i;
+    int i, j;
     char buf[BUFSIZ];
 
     if (position) *position = ftell(fp);
     cp = fgets(line, size, fp);
     if (cp == NULL) line[0] = '\0';
+    for(i = 0; line[i] != '\0' && (line[i] == 0x0c || line[i] == ' ' || line[i] == '\t'); i++) 
+            ;
+    if(i > 0 && line[i] == '%' && line[i + 1] == '%') {
+            for(j = i; line[j] != '\0'; j++)
+                    line[j - i] = line[j];
+            line[j - i] = '\0';
+    }
     *line_len = strlen(line);
     if (!(DSCcomment(line) && iscomment(line+2, "Begin"))) {
 	/* Do nothing */
@@ -1395,7 +1402,7 @@ readline(line, size, fp, position, line_len)
     } else if (iscomment(line+7, "Data:")) {
 	text[0] = '\0';
 	strcpy(save, line+7);
-	if (sscanf(line+length("%%BeginData:"), "%d %*s %s", &num, text) >= 1) {
+	if (sscanf(line+length("%%BeginData:"), "%d %*s %256s", &num, text) >= 1) {
 	    if (strcmp(text, "Lines") == 0) {
 		for (i=0; i < num; i++) {
 		    cp = fgets(line, size, fp);
@@ -1470,7 +1477,7 @@ pscopy(from, to, begin, end)
                 } else if (iscomment(line+7, "Data:")) {
                         text[0] = '\0';
                         if (sscanf(line+length("%%BeginData:"),
-                                   "%d %*s %s", &num, text) >= 1) {
+                                   "%d %*s %256s", &num, text) >= 1) {
                                 if (strcmp(text, "Lines") == 0) {
                                         for (i=0; i < num; i++) {
                                                 fgets(line, sizeof(line), from);
@@ -1539,7 +1546,7 @@ pscopyuntil(FILE *from, GtkGSDocSink *to, long begin, long end,
                 } else if (iscomment(line+7, "Data:")) {
                         text[0] = '\0';
                         if (sscanf(line+length("%%BeginData:"),
-                                   "%d %*s %s", &num, text) >= 1) {
+                                   "%d %*s %256s", &num, text) >= 1) {
                                 if (strcmp(text, "Lines") == 0) {
                                         for (i=0; i < num; i++) {
                                                 fgets(line, sizeof line, from);
@@ -1617,7 +1624,7 @@ void pscopydoc(GtkGSDocSink *dest,
                         g_free(comment);
                         continue;
                 }
-                sscanf(comment+length("%%Pages:"), "%s", text);
+                sscanf(comment+length("%%Pages:"), "%256s", text);
                 if (strcmp(text, "(atend)") == 0) {
                         gtk_gs_doc_sink_write(dest, comment, strlen(comment));
                         pages_atend = TRUE;
